#!/usr/bin/env python3
import common

import argparse
import sys
from gitlab import Gitlab, GitlabGetError


def retrigger_mr_pipeline_job(project, mr, job_name, status_list):
    # Get pipeline
    if not mr.pipeline:
        print("No pipeline in !%s" % mr.iid)
        return None
    pipeline = project.pipelines.get(mr.pipeline.get("id"), retry_transient_errors=True)

    # Find job
    job = None
    for pipelinejob in pipeline.jobs.list():
        if pipelinejob.name == job_name:
            job = project.jobs.get(pipelinejob.id, retry_transient_errors=True)
    if not job:
        print("Could not find job '%s' in pipeline of !%s" % (job_name, mr.iid))
        return None

    # Only retrigger if job is in certain status
    if job.status not in status_list:
        return None

    # Retrigger job
    job.retry()
    print("Retrigger job '%s' of pipeline #%s:" % (job_name, pipeline.id))
    job = project.jobs.get(job.id, retry_transient_errors=True)
    print(job.web_url)

    return job


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--gitlab-url",
        help="""URL to the GitLab instance""",
        dest="gitlab_url",
        required=True,
    )
    parser.add_argument(
        "--token",
        help="""GitLab REST API private access token""",
        dest="token",
        required=True,
    )
    parser.add_argument(
        "--project",
        help="""name of the GitLab project""",
        dest="project",
        required=True,
    )
    parser.add_argument(
        "--iid",
        help="""iid of the merge request""",
        dest="iid",
        required=True,
    )
    parser.add_argument(
        "--job",
        help="""name of the job""",
        dest="job",
        required=True,
    )

    args, _ = parser.parse_known_args()

    gitlab = Gitlab(args.gitlab_url, private_token=args.token)
    project = common.get_project(gitlab, args.project)
    try:
        mr = project.mergerequests.get(args.iid, retry_transient_errors=True)
    except GitlabGetError as e:
        sys.exit(
            "ERROR: could not get merge request %s:!%s: %s"
            % (project.name, args.iid, e)
        )
    if not mr:
        sys.exit("ERROR: could not find merge request %s!%s" % (project.name, args.iid))

    retrigger_mr_pipeline_job(project, mr, args.job, ["success", "running"])


if __name__ == "__main__":
    main()
